@@ -1,10 +1,11 @@
 
 normalize <- function(las, normalized = NULL,
                       x.center = NULL, y.center = NULL,
+                      x.side = NULL, y.side = NULL,
                       max.dist = NULL, min.height = NULL, max.height = 50,
                       algorithm.dtm = "knnidw", res.dtm = 0.2,
                       csf = list(cloth_resolution = 0.5),
-                      RGB = NULL,
+                      intensity = NULL, RGB = NULL,
                       scan.approach = "single",
                       id = NULL, file = NULL,
                       dir.data = NULL, save.result = TRUE, dir.result = NULL){
@@ -27,9 +28,14 @@ normalize <- function(las, normalized = NULL,
 
   # Loading input (LAS file)
 
-  if(is.null(RGB)){
-  .las <- suppressWarnings(suppressMessages(lidR::readLAS(file.path(dir.data, las), select = "xyz")))} else {
-    .las <- suppressWarnings(suppressMessages(lidR::readLAS(file.path(dir.data, las), select = "xyzRGB")))}
+  if(is.null(RGB) & is.null(intensity)){
+  .las <- suppressWarnings(suppressMessages(lidR::readLAS(file.path(dir.data, las), select = "xyz")))}
+    else if (is.null(RGB) & !is.null(intensity)){
+      .las <- suppressWarnings(suppressMessages(lidR::readLAS(file.path(dir.data, las), select = "xyzIntensity")))}
+        else if (!is.null(RGB) & is.null(intensity)){
+          .las <- suppressWarnings(suppressMessages(lidR::readLAS(file.path(dir.data, las), select = "xyzRGB")))}
+            else{
+              .las <- suppressWarnings(suppressMessages(lidR::readLAS(file.path(dir.data, las), select = "xyzIntensityRGB")))}
 
 
   .pb$tick()
@@ -63,9 +69,18 @@ normalize <- function(las, normalized = NULL,
 
   if(!is.null(normalized)) {
 
-    if(!is.null(max.dist)){
+      if(!is.null(max.dist)){
+
       .data <- lidR::clip_circle(.las, 0, 0, max.dist)
-      .data <- data.frame(.data@data)} else {
+      .data <- data.frame(.data@data)}
+
+      else if (!is.null(x.side) | !is.null(y.side)){
+
+      .data <- lidR::clip_rectangle(.las, 0 - (x.side / 2), 0 - (y.side / 2),
+                                          0 + (x.side / 2), 0 + (y.side / 2))
+      .data <- data.frame(.data@data)
+
+      } else {
 
       .data <- data.frame(.las@data)}
 
@@ -150,11 +165,16 @@ normalize <- function(las, normalized = NULL,
 
   # Data filtering at horizontal distances larger than max_dist m in the horizontal plane
 
-  if(!is.null(max.dist)) {
+  if(!is.null(max.dist)){
 
     .data <- lidR::clip_circle(.data, 0, 0, max.dist)
 
-    }
+    } else if (!is.null(x.side) | !is.null(y.side)){
+
+    .data <- lidR::clip_rectangle(.data, 0 - (x.side / 2), 0 - (y.side / 2),
+                                  0 + (x.side / 2), 0 + (y.side / 2))
+
+  }
 
   .pb$tick()
 
@@ -181,11 +201,18 @@ normalize <- function(las, normalized = NULL,
 
   # Extracting coordinates values
 
-  if(is.null(RGB)){
-  .data <- .data[, c("X", "Y", "Z", "slope"), drop = FALSE]
-  colnames(.data) <- c("x", "y", "z", "slope")} else {
+  if(is.null(RGB) & is.null(intensity)){
+    .data <- .data[, c("X", "Y", "Z", "slope"), drop = FALSE]
+    colnames(.data) <- c("x", "y", "z", "slope")}
+  else if (is.null(RGB) & !is.null(intensity)){
+    .data <- .data[, c("X", "Y", "Z", "slope", "Intensity"), drop = FALSE]
+    colnames(.data) <- c("x", "y", "z", "slope", "intensity")}
+  else if (!is.null(RGB) & is.null(intensity)){
     .data <- .data[, c("X", "Y", "Z", "slope", "R", "G", "B"), drop = FALSE]
     colnames(.data) <- c("x", "y", "z", "slope", "R", "G", "B")}
+  else{
+    .data <- .data[, c("X", "Y", "Z", "slope", "Intensity", "R", "G", "B"), drop = FALSE]
+    colnames(.data) <- c("x", "y", "z", "slope", "intensity", "R", "G", "B")}
 
 
   # Low point filtering
@@ -246,7 +273,7 @@ normalize <- function(las, normalized = NULL,
   if(scan.approach == "multi"){
 
     .data$prob <- stats::runif(nrow(.data))
-    .data$prob.selec <- ifelse(.data$prob >= 0.75, 1, 0)}
+    .data$prob.selec <- ifelse(.data$prob >= 0.5, 1, 0)}
 
   # Assign id
 
@@ -273,9 +300,13 @@ normalize <- function(las, normalized = NULL,
 
   }
 
-  if(is.null(RGB)){
-  .data <- .data[, c("id", "file", "point", "x", "y", "z", "rho", "phi", "r", "theta", "slope", "prob", "prob.selec"), drop = FALSE]} else {
-    .data <- .data[, c("id", "file", "point", "x", "y", "z", "rho", "phi", "r", "theta", "slope", "GLA", "prob", "prob.selec"), drop = FALSE]}
+  if(is.null(RGB) & is.null(intensity)){
+    .data <- .data[, c("id", "file", "point", "x", "y", "z", "rho", "phi", "r", "theta", "slope", "prob", "prob.selec"), drop = FALSE]}
+  else if (is.null(RGB) & !is.null(intensity)){
+    .data <- .data[, c("id", "file", "point", "x", "y", "z", "rho", "phi", "r", "theta", "slope", "intensity", "prob", "prob.selec"), drop = FALSE]}
+  else if (!is.null(RGB) & is.null(intensity)){
+    .data <- .data[, c("id", "file", "point", "x", "y", "z", "rho", "phi", "r", "theta", "slope", "R", "G", "B", "GLA", "prob", "prob.selec"), drop = FALSE]}
+  else{.data <- .data[, c("id", "file", "point", "x", "y", "z", "rho", "phi", "r", "theta", "slope", "intensity", "R", "G", "B", "GLA", "prob", "prob.selec"), drop = FALSE]}
 
   .pb$tick()
 
@@ -295,6 +326,4 @@ normalize <- function(las, normalized = NULL,
 
   return(.data)
 
-
-
 }
